"""
Helpers for asset management tasks.

"""

import typing
import collections.abc

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class AssetBrowserPanel:
    bl_space_type: typing.Any

    def asset_browser_panel_poll(self, context):
        """

        :param context:
        """
        ...

    def poll(self, context):
        """

        :param context:
        """
        ...

class AssetMetaDataPanel:
    bl_region_type: typing.Any
    bl_space_type: typing.Any

    def poll(self, context):
        """

        :param context:
        """
        ...

class SpaceAssetInfo:
    def is_asset_browser(self, space_data):
        """

        :param space_data:
        """
        ...

    def is_asset_browser_poll(self, context):
        """

        :param context:
        """
        ...



import typing
import collections.abc
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def autocomplete(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Show a list of used text in the open document

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def comment_toggle(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    type: str | None = "TOGGLE",
):
    """Undocumented, consider contributing.

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param type: Type, Add or remove comments
    :type type: str | None
    """

    ...

def convert_whitespace(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    type: str | None = "SPACES",
):
    """Convert whitespaces by type

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param type: Type, Type of whitespace to convert to
    :type type: str | None
    """

    ...

def copy(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Copy selected text to clipboard

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def cursor_set(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    x: typing.Any | None = 0,
    y: typing.Any | None = 0,
):
    """Set cursor position

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param x: X
    :type x: typing.Any | None
    :param y: Y
    :type y: typing.Any | None
    """

    ...

def cut(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Cut selected text to clipboard

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def delete(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    type: str | None = "NEXT_CHARACTER",
):
    """Delete text by cursor position

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param type: Type, Which part of the text to delete
    :type type: str | None
    """

    ...

def duplicate_line(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Duplicate the current line

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def find(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Find specified text

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def find_set_selected(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Find specified text and set as selected

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def indent(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Indent selected text

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def indent_or_autocomplete(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Indent selected text or autocomplete

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def insert(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    text: str | typing.Any = "",
):
    """Insert text at cursor position

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param text: Text, Text to insert at the cursor position
    :type text: str | typing.Any
    """

    ...

def jump(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    line: typing.Any | None = 1,
):
    """Jump cursor to line

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param line: Line, Line number to jump to
    :type line: typing.Any | None
    """

    ...

def jump_to_file_at_point(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    filepath: str | typing.Any = "",
    line: typing.Any | None = 0,
    column: typing.Any | None = 0,
):
    """Jump to a file for the text editor

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param filepath: Filepath
    :type filepath: str | typing.Any
    :param line: Line, Line to jump to
    :type line: typing.Any | None
    :param column: Column, Column to jump to
    :type column: typing.Any | None
    """

    ...

def line_break(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Insert line break at cursor position

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def line_number(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """The current line number

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def make_internal(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Make active text file internal

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def move(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    type: str | None = "LINE_BEGIN",
):
    """Move cursor to position type

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param type: Type, Where to move cursor to
    :type type: str | None
    """

    ...

def move_lines(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    direction: str | None = "DOWN",
):
    """Move the currently selected line(s) up/down

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param direction: Direction
    :type direction: str | None
    """

    ...

def move_select(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    type: str | None = "LINE_BEGIN",
):
    """Move the cursor while selecting

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param type: Type, Where to move cursor to, to make a selection
    :type type: str | None
    """

    ...

def new(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Create a new text data-block

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def open(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    filepath: str | typing.Any = "",
    hide_props_region: bool | typing.Any | None = True,
    check_existing: bool | typing.Any | None = False,
    filter_blender: bool | typing.Any | None = False,
    filter_backup: bool | typing.Any | None = False,
    filter_image: bool | typing.Any | None = False,
    filter_movie: bool | typing.Any | None = False,
    filter_python: bool | typing.Any | None = True,
    filter_font: bool | typing.Any | None = False,
    filter_sound: bool | typing.Any | None = False,
    filter_text: bool | typing.Any | None = True,
    filter_archive: bool | typing.Any | None = False,
    filter_btx: bool | typing.Any | None = False,
    filter_collada: bool | typing.Any | None = False,
    filter_alembic: bool | typing.Any | None = False,
    filter_usd: bool | typing.Any | None = False,
    filter_obj: bool | typing.Any | None = False,
    filter_volume: bool | typing.Any | None = False,
    filter_folder: bool | typing.Any | None = True,
    filter_blenlib: bool | typing.Any | None = False,
    filemode: typing.Any | None = 9,
    display_type: str | None = "DEFAULT",
    sort_method: str | None = "",
    internal: bool | typing.Any | None = False,
):
    """Open a new text data-block

        :type override_context: bpy.types.Context | dict[str, typing.Any] | None
        :type execution_context: int | str | None
        :type undo: bool | None
        :param filepath: File Path, Path to file
        :type filepath: str | typing.Any
        :param hide_props_region: Hide Operator Properties, Collapse the region displaying the operator settings
        :type hide_props_region: bool | typing.Any | None
        :param check_existing: Check Existing, Check and warn on overwriting existing files
        :type check_existing: bool | typing.Any | None
        :param filter_blender: Filter .blend files
        :type filter_blender: bool | typing.Any | None
        :param filter_backup: Filter .blend files
        :type filter_backup: bool | typing.Any | None
        :param filter_image: Filter image files
        :type filter_image: bool | typing.Any | None
        :param filter_movie: Filter movie files
        :type filter_movie: bool | typing.Any | None
        :param filter_python: Filter Python files
        :type filter_python: bool | typing.Any | None
        :param filter_font: Filter font files
        :type filter_font: bool | typing.Any | None
        :param filter_sound: Filter sound files
        :type filter_sound: bool | typing.Any | None
        :param filter_text: Filter text files
        :type filter_text: bool | typing.Any | None
        :param filter_archive: Filter archive files
        :type filter_archive: bool | typing.Any | None
        :param filter_btx: Filter btx files
        :type filter_btx: bool | typing.Any | None
        :param filter_collada: Filter COLLADA files
        :type filter_collada: bool | typing.Any | None
        :param filter_alembic: Filter Alembic files
        :type filter_alembic: bool | typing.Any | None
        :param filter_usd: Filter USD files
        :type filter_usd: bool | typing.Any | None
        :param filter_obj: Filter OBJ files
        :type filter_obj: bool | typing.Any | None
        :param filter_volume: Filter OpenVDB volume files
        :type filter_volume: bool | typing.Any | None
        :param filter_folder: Filter folders
        :type filter_folder: bool | typing.Any | None
        :param filter_blenlib: Filter Blender IDs
        :type filter_blenlib: bool | typing.Any | None
        :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file
        :type filemode: typing.Any | None
        :param display_type: Display Type

    DEFAULT
    Default -- Automatically determine display type for files.

    LIST_VERTICAL
    Short List -- Display files as short list.

    LIST_HORIZONTAL
    Long List -- Display files as a detailed list.

    THUMBNAIL
    Thumbnails -- Display files as thumbnails.
        :type display_type: str | None
        :param sort_method: File sorting mode

    DEFAULT
    Default -- Automatically determine sort method for files.

    FILE_SORT_ALPHA
    Name -- Sort the file list alphabetically.

    FILE_SORT_EXTENSION
    Extension -- Sort the file list by extension/type.

    FILE_SORT_TIME
    Modified Date -- Sort files by modification time.

    FILE_SORT_SIZE
    Size -- Sort files by size.
        :type sort_method: str | None
        :param internal: Make Internal, Make text file internal after loading
        :type internal: bool | typing.Any | None
    """

    ...

def overwrite_toggle(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Toggle overwrite while typing

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def paste(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    selection: bool | typing.Any | None = False,
):
    """Paste text from clipboard

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param selection: Selection, Paste text selected elsewhere rather than copied (X11/Wayland only)
    :type selection: bool | typing.Any | None
    """

    ...

def refresh_pyconstraints(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Refresh all pyconstraints

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def reload(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Reload active text data-block from its file

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def replace(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    all: bool | typing.Any | None = False,
):
    """Replace text with the specified text

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param all: Replace All, Replace all occurrences
    :type all: bool | typing.Any | None
    """

    ...

def replace_set_selected(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Replace text with specified text and set as selected

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def resolve_conflict(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    resolution: str | None = "IGNORE",
):
    """When external text is out of sync, resolve the conflict

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param resolution: Resolution, How to solve conflict due to differences in internal and external text
    :type resolution: str | None
    """

    ...

def run_script(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Run active script

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def save(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Save active text data-block

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def save_as(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    filepath: str | typing.Any = "",
    hide_props_region: bool | typing.Any | None = True,
    check_existing: bool | typing.Any | None = True,
    filter_blender: bool | typing.Any | None = False,
    filter_backup: bool | typing.Any | None = False,
    filter_image: bool | typing.Any | None = False,
    filter_movie: bool | typing.Any | None = False,
    filter_python: bool | typing.Any | None = True,
    filter_font: bool | typing.Any | None = False,
    filter_sound: bool | typing.Any | None = False,
    filter_text: bool | typing.Any | None = True,
    filter_archive: bool | typing.Any | None = False,
    filter_btx: bool | typing.Any | None = False,
    filter_collada: bool | typing.Any | None = False,
    filter_alembic: bool | typing.Any | None = False,
    filter_usd: bool | typing.Any | None = False,
    filter_obj: bool | typing.Any | None = False,
    filter_volume: bool | typing.Any | None = False,
    filter_folder: bool | typing.Any | None = True,
    filter_blenlib: bool | typing.Any | None = False,
    filemode: typing.Any | None = 9,
    display_type: str | None = "DEFAULT",
    sort_method: str | None = "",
):
    """Save active text file with options

        :type override_context: bpy.types.Context | dict[str, typing.Any] | None
        :type execution_context: int | str | None
        :type undo: bool | None
        :param filepath: File Path, Path to file
        :type filepath: str | typing.Any
        :param hide_props_region: Hide Operator Properties, Collapse the region displaying the operator settings
        :type hide_props_region: bool | typing.Any | None
        :param check_existing: Check Existing, Check and warn on overwriting existing files
        :type check_existing: bool | typing.Any | None
        :param filter_blender: Filter .blend files
        :type filter_blender: bool | typing.Any | None
        :param filter_backup: Filter .blend files
        :type filter_backup: bool | typing.Any | None
        :param filter_image: Filter image files
        :type filter_image: bool | typing.Any | None
        :param filter_movie: Filter movie files
        :type filter_movie: bool | typing.Any | None
        :param filter_python: Filter Python files
        :type filter_python: bool | typing.Any | None
        :param filter_font: Filter font files
        :type filter_font: bool | typing.Any | None
        :param filter_sound: Filter sound files
        :type filter_sound: bool | typing.Any | None
        :param filter_text: Filter text files
        :type filter_text: bool | typing.Any | None
        :param filter_archive: Filter archive files
        :type filter_archive: bool | typing.Any | None
        :param filter_btx: Filter btx files
        :type filter_btx: bool | typing.Any | None
        :param filter_collada: Filter COLLADA files
        :type filter_collada: bool | typing.Any | None
        :param filter_alembic: Filter Alembic files
        :type filter_alembic: bool | typing.Any | None
        :param filter_usd: Filter USD files
        :type filter_usd: bool | typing.Any | None
        :param filter_obj: Filter OBJ files
        :type filter_obj: bool | typing.Any | None
        :param filter_volume: Filter OpenVDB volume files
        :type filter_volume: bool | typing.Any | None
        :param filter_folder: Filter folders
        :type filter_folder: bool | typing.Any | None
        :param filter_blenlib: Filter Blender IDs
        :type filter_blenlib: bool | typing.Any | None
        :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file
        :type filemode: typing.Any | None
        :param display_type: Display Type

    DEFAULT
    Default -- Automatically determine display type for files.

    LIST_VERTICAL
    Short List -- Display files as short list.

    LIST_HORIZONTAL
    Long List -- Display files as a detailed list.

    THUMBNAIL
    Thumbnails -- Display files as thumbnails.
        :type display_type: str | None
        :param sort_method: File sorting mode
        :type sort_method: str | None
    """

    ...

def scroll(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    lines: typing.Any | None = 1,
):
    """Undocumented, consider contributing.

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param lines: Lines, Number of lines to scroll
    :type lines: typing.Any | None
    """

    ...

def scroll_bar(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    lines: typing.Any | None = 1,
):
    """Undocumented, consider contributing.

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param lines: Lines, Number of lines to scroll
    :type lines: typing.Any | None
    """

    ...

def select_all(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Select all text

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def select_line(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Select text by line

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def select_word(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Select word under cursor

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def selection_set(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Set text selection

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def start_find(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Start searching text

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def to_3d_object(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    split_lines: bool | typing.Any | None = False,
):
    """Create 3D text object from active text data-block

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param split_lines: Split Lines, Create one object per line in the text
    :type split_lines: bool | typing.Any | None
    """

    ...

def unindent(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Unindent selected text

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...

def unlink(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Unlink active text data-block

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...



"""
This module is used for all Blender/Python access.

```../examples/bpy.data.py```

"""

import typing
import collections.abc
import bpy.types

from . import app
from . import msgbus
from . import ops
from . import path
from . import props
from . import types
from . import utils

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")
context: bpy.types.Context

data: bpy.types.BlendData
""" Access to Blender's internal data
"""



"""
This module contains functions operating on vertices (0D elements) and
polylines (1D elements).  The module is also intended to be a
collection of examples for function definition in Python.

User-defined functions inherit one of the following base classes,
depending on the object type (0D or 1D) to operate on and the return
value type:

* freestyle.types.UnaryFunction0DDouble
* freestyle.types.UnaryFunction0DEdgeNature
* freestyle.types.UnaryFunction0DFloat
* freestyle.types.UnaryFunction0DId
* freestyle.types.UnaryFunction0DMaterial
* freestyle.types.UnaryFunction0DUnsigned
* freestyle.types.UnaryFunction0DVec2f
* freestyle.types.UnaryFunction0DVec3f
* freestyle.types.UnaryFunction0DVectorViewShape
* freestyle.types.UnaryFunction0DViewShape
* freestyle.types.UnaryFunction1DDouble
* freestyle.types.UnaryFunction1DEdgeNature
* freestyle.types.UnaryFunction1DFloat
* freestyle.types.UnaryFunction1DUnsigned
* freestyle.types.UnaryFunction1DVec2f
* freestyle.types.UnaryFunction1DVec3f
* freestyle.types.UnaryFunction1DVectorViewShape
* freestyle.types.UnaryFunction1DVoid

"""

import typing
import collections.abc
import freestyle.types
import mathutils

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class ChainingTimeStampF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DVoid` > `ChainingTimeStampF1D`"""

    def __init__(self):
        """Builds a ChainingTimeStampF1D object."""
        ...

    def __call__(self, inter: freestyle.types.Interface1D):
        """Sets the chaining time stamp of the Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        """
        ...

class Curvature2DAngleF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `Curvature2DAngleF0D`"""

    def __init__(self):
        """Builds a Curvature2DAngleF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns a real value giving the 2D curvature (as an angle) of the 1D
        element to which the `freestyle.types.Interface0D` pointed by
        the Interface0DIterator belongs. The 2D curvature is evaluated at the
        Interface0D.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The 2D curvature of the 1D element evaluated at the
        pointed Interface0D.
                :rtype: float
        """
        ...

class Curvature2DAngleF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `Curvature2DAngleF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a Curvature2DAngleF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the 2D curvature as an angle for an Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The 2D curvature as an angle.
        :rtype: float
        """
        ...

class CurveMaterialF0D:
    """A replacement of the built-in MaterialF0D for stroke creation.
    MaterialF0D does not work with Curves and Strokes.  Line color
    priority is used to pick one of the two materials at material
    boundaries.
    """

    ...

class CurveNatureF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DEdgeNature` > `CurveNatureF0D`"""

    def __init__(self):
        """Builds a CurveNatureF0D object."""
        ...

    def __call__(
        self, it: freestyle.types.Interface0DIterator
    ) -> freestyle.types.Nature:
        """Returns the `freestyle.types.Nature` of the 1D element the
        Interface0D pointed by the Interface0DIterator belongs to.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The nature of the 1D element to which the pointed Interface0D
        belongs.
                :rtype: freestyle.types.Nature
        """
        ...

class CurveNatureF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DEdgeNature` > `CurveNatureF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a CurveNatureF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> freestyle.types.Nature:
        """Returns the nature of the Interface1D (silhouette, ridge, crease, and
        so on). Except if the Interface1D is a
        `freestyle.types.ViewEdge`, this result might be ambiguous.
        Indeed, the Interface1D might result from the gathering of several 1D
        elements, each one being of a different nature. An integration
        method, such as the MEAN, might give, in this case, irrelevant
        results.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: The nature of the Interface1D.
                :rtype: freestyle.types.Nature
        """
        ...

class DensityF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `DensityF0D`"""

    def __init__(self, sigma: float = 2.0):
        """Builds a DensityF0D object.

                :param sigma: The gaussian sigma value indicating the X value for
        which the gaussian function is 0.5. It leads to the window size
        value (the larger, the smoother).
                :type sigma: float
        """
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the density of the (result) image evaluated at the
        `freestyle.types.Interface0D` pointed by the
        Interface0DIterator. This density is evaluated using a pixels square
        window around the evaluation point and integrating these values using
        a gaussian.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The density of the image evaluated at the pointed
        Interface0D.
                :rtype: float
        """
        ...

class DensityF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `DensityF1D`"""

    def __init__(
        self,
        sigma: float = 2.0,
        integration_type: freestyle.types.IntegrationType = None,
        sampling: float = 2.0,
    ):
        """Builds a DensityF1D object.

                :param sigma: The sigma used in DensityF0D and determining the window size
        used in each density query.
                :type sigma: float
                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
                :param sampling: The resolution used to sample the chain: the
        corresponding 0D function is evaluated at each sample point and
        the result is obtained by combining the resulting values into a
        single one, following the method specified by integration_type.
                :type sampling: float
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the density evaluated for an Interface1D. The density is
        evaluated for a set of points along the Interface1D (using the
        `freestyle.functions.DensityF0D` functor) with a user-defined
        sampling and then integrated into a single value using a user-defined
        integration method.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: The density evaluated for an Interface1D.
                :rtype: float
        """
        ...

class GetCompleteViewMapDensityF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetCompleteViewMapDensityF1D`"""

    def __init__(
        self,
        level: int,
        integration_type: freestyle.types.IntegrationType = None,
        sampling: float = 2.0,
    ):
        """Builds a GetCompleteViewMapDensityF1D object.

                :param level: The level of the pyramid from which the pixel must be
        read.
                :type level: int
                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
                :param sampling: The resolution used to sample the chain: the
        corresponding 0D function is evaluated at each sample point and
        the result is obtained by combining the resulting values into a
        single one, following the method specified by integration_type.
                :type sampling: float
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the density evaluated for an Interface1D in the complete
        viewmap image. The density is evaluated for a set of points along the
        Interface1D (using the
        `freestyle.functions.ReadCompleteViewMapPixelF0D` functor) and
        then integrated into a single value using a user-defined integration
        method.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: The density evaluated for the Interface1D in the complete
        viewmap image.
                :rtype: float
        """
        ...

class GetCurvilinearAbscissaF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DFloat` > `GetCurvilinearAbscissaF0D`"""

    def __init__(self):
        """Builds a GetCurvilinearAbscissaF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the curvilinear abscissa of the
        `freestyle.types.Interface0D` pointed by the
        Interface0DIterator in the context of its 1D element.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The curvilinear abscissa of the pointed Interface0D.
                :rtype: float
        """
        ...

class GetDirectionalViewMapDensityF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetDirectionalViewMapDensityF1D`"""

    def __init__(
        self,
        orientation: int,
        level: int,
        integration_type: freestyle.types.IntegrationType = None,
        sampling: float = 2.0,
    ):
        """Builds a GetDirectionalViewMapDensityF1D object.

                :param orientation: The number of the directional map we must work
        with.
                :type orientation: int
                :param level: The level of the pyramid from which the pixel must be
        read.
                :type level: int
                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
                :param sampling: The resolution used to sample the chain: the
        corresponding 0D function is evaluated at each sample point and
        the result is obtained by combining the resulting values into a
        single one, following the method specified by integration_type.
                :type sampling: float
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the density evaluated for an Interface1D in of the steerable
        viewmaps image. The direction telling which Directional map to choose
        is explicitly specified by the user. The density is evaluated for a
        set of points along the Interface1D (using the
        `freestyle.functions.ReadSteerableViewMapPixelF0D` functor) and
        then integrated into a single value using a user-defined integration
        method.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: the density evaluated for an Interface1D in of the
        steerable viewmaps image.
                :rtype: float
        """
        ...

class GetOccludeeF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DViewShape` > `GetOccludeeF0D`"""

    def __init__(self):
        """Builds a GetOccludeeF0D object."""
        ...

    def __call__(
        self, it: freestyle.types.Interface0DIterator
    ) -> freestyle.types.ViewShape:
        """Returns the `freestyle.types.ViewShape` that the Interface0D
        pointed by the Interface0DIterator occludes.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The ViewShape occluded by the pointed Interface0D.
                :rtype: freestyle.types.ViewShape
        """
        ...

class GetOccludeeF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DVectorViewShape` > `GetOccludeeF1D`"""

    def __init__(self):
        """Builds a GetOccludeeF1D object."""
        ...

    def __call__(self, inter: freestyle.types.Interface1D):
        """Returns a list of occluded shapes covered by this Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: A list of occluded shapes covered by the Interface1D.
        """
        ...

class GetOccludersF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DVectorViewShape` > `GetOccludersF0D`"""

    def __init__(self):
        """Builds a GetOccludersF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator):
        """Returns a list of `freestyle.types.ViewShape` objects occluding the
        `freestyle.types.Interface0D` pointed by the Interface0DIterator.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: A list of ViewShape objects occluding the pointed
        Interface0D.
        """
        ...

class GetOccludersF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DVectorViewShape` > `GetOccludersF1D`"""

    def __init__(self):
        """Builds a GetOccludersF1D object."""
        ...

    def __call__(self, inter: freestyle.types.Interface1D):
        """Returns a list of occluding shapes that cover this Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: A list of occluding shapes that cover the Interface1D.
        """
        ...

class GetParameterF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DFloat` > `GetParameterF0D`"""

    def __init__(self):
        """Builds a GetParameterF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the parameter of the `freestyle.types.Interface0D`
        pointed by the Interface0DIterator in the context of its 1D element.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The parameter of an Interface0D.
                :rtype: float
        """
        ...

class GetProjectedXF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `GetProjectedXF0D`"""

    def __init__(self):
        """Builds a GetProjectedXF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the X 3D projected coordinate of the `freestyle.types.Interface0D`
        pointed by the Interface0DIterator.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The X 3D projected coordinate of the pointed Interface0D.
                :rtype: float
        """
        ...

class GetProjectedXF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetProjectedXF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a GetProjectedXF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the projected X 3D coordinate of an Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The projected X 3D coordinate of an Interface1D.
        :rtype: float
        """
        ...

class GetProjectedYF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `GetProjectedYF0D`"""

    def __init__(self):
        """Builds a GetProjectedYF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the Y 3D projected coordinate of the `freestyle.types.Interface0D`
        pointed by the Interface0DIterator.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The Y 3D projected coordinate of the pointed Interface0D.
                :rtype: float
        """
        ...

class GetProjectedYF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetProjectedYF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a GetProjectedYF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the projected Y 3D coordinate of an Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The projected Y 3D coordinate of an Interface1D.
        :rtype: float
        """
        ...

class GetProjectedZF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `GetProjectedZF0D`"""

    def __init__(self):
        """Builds a GetProjectedZF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the Z 3D projected coordinate of the `freestyle.types.Interface0D`
        pointed by the Interface0DIterator.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The Z 3D projected coordinate of the pointed Interface0D.
                :rtype: float
        """
        ...

class GetProjectedZF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetProjectedZF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a GetProjectedZF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the projected Z 3D coordinate of an Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The projected Z 3D coordinate of an Interface1D.
        :rtype: float
        """
        ...

class GetShapeF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DViewShape` > `GetShapeF0D`"""

    def __init__(self):
        """Builds a GetShapeF0D object."""
        ...

    def __call__(
        self, it: freestyle.types.Interface0DIterator
    ) -> freestyle.types.ViewShape:
        """Returns the `freestyle.types.ViewShape` containing the
        Interface0D pointed by the Interface0DIterator.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The ViewShape containing the pointed Interface0D.
                :rtype: freestyle.types.ViewShape
        """
        ...

class GetShapeF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DVectorViewShape` > `GetShapeF1D`"""

    def __init__(self):
        """Builds a GetShapeF1D object."""
        ...

    def __call__(self, inter: freestyle.types.Interface1D):
        """Returns a list of shapes covered by this Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: A list of shapes covered by the Interface1D.
        """
        ...

class GetSteerableViewMapDensityF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetSteerableViewMapDensityF1D`"""

    def __init__(
        self,
        level: int,
        integration_type: freestyle.types.IntegrationType = None,
        sampling: float = 2.0,
    ):
        """Builds a GetSteerableViewMapDensityF1D object.

                :param level: The level of the pyramid from which the pixel must be
        read.
                :type level: int
                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
                :param sampling: The resolution used to sample the chain: the
        corresponding 0D function is evaluated at each sample point and
        the result is obtained by combining the resulting values into a
        single one, following the method specified by integration_type.
                :type sampling: float
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the density of the ViewMap for a given Interface1D. The
        density of each `freestyle.types.FEdge` is evaluated in the
        proper steerable `freestyle.types.ViewMap` depending on its
        orientation.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: The density of the ViewMap for a given Interface1D.
                :rtype: float
        """
        ...

class GetViewMapGradientNormF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DFloat` > `GetViewMapGradientNormF0D`"""

    def __init__(self, level: int):
        """Builds a GetViewMapGradientNormF0D object.

                :param level: The level of the pyramid from which the pixel must be
        read.
                :type level: int
        """
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the norm of the gradient of the global viewmap density
        image.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The norm of the gradient of the global viewmap density
        image.
                :rtype: float
        """
        ...

class GetViewMapGradientNormF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetViewMapGradientNormF1D`"""

    def __init__(
        self,
        level: int,
        integration_type: freestyle.types.IntegrationType = None,
        sampling: float = 2.0,
    ):
        """Builds a GetViewMapGradientNormF1D object.

                :param level: The level of the pyramid from which the pixel must be
        read.
                :type level: int
                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
                :param sampling: The resolution used to sample the chain: the
        corresponding 0D function is evaluated at each sample point and
        the result is obtained by combining the resulting values into a
        single one, following the method specified by integration_type.
                :type sampling: float
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the density of the ViewMap for a given Interface1D. The
        density of each `freestyle.types.FEdge` is evaluated in the
        proper steerable `freestyle.types.ViewMap` depending on its
        orientation.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: The density of the ViewMap for a given Interface1D.
                :rtype: float
        """
        ...

class GetXF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `GetXF0D`"""

    def __init__(self):
        """Builds a GetXF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the X 3D coordinate of the `freestyle.types.Interface0D` pointed by
        the Interface0DIterator.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The X 3D coordinate of the pointed Interface0D.
                :rtype: float
        """
        ...

class GetXF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetXF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a GetXF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the X 3D coordinate of an Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The X 3D coordinate of the Interface1D.
        :rtype: float
        """
        ...

class GetYF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `GetYF0D`"""

    def __init__(self):
        """Builds a GetYF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the Y 3D coordinate of the `freestyle.types.Interface0D` pointed by
        the Interface0DIterator.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The Y 3D coordinate of the pointed Interface0D.
                :rtype: float
        """
        ...

class GetYF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetYF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a GetYF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the Y 3D coordinate of an Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The Y 3D coordinate of the Interface1D.
        :rtype: float
        """
        ...

class GetZF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `GetZF0D`"""

    def __init__(self):
        """Builds a GetZF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the Z 3D coordinate of the `freestyle.types.Interface0D` pointed by
        the Interface0DIterator.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The Z 3D coordinate of the pointed Interface0D.
                :rtype: float
        """
        ...

class GetZF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `GetZF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a GetZF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the Z 3D coordinate of an Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The Z 3D coordinate of the Interface1D.
        :rtype: float
        """
        ...

class IncrementChainingTimeStampF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DVoid` > `IncrementChainingTimeStampF1D`"""

    def __init__(self):
        """Builds an IncrementChainingTimeStampF1D object."""
        ...

    def __call__(self, inter: freestyle.types.Interface1D):
        """Increments the chaining time stamp of the Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        """
        ...

class LocalAverageDepthF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `LocalAverageDepthF0D`"""

    def __init__(self, mask_size: float = 5.0):
        """Builds a LocalAverageDepthF0D object.

        :param mask_size: The size of the mask.
        :type mask_size: float
        """
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns the average depth around the
        `freestyle.types.Interface0D` pointed by the
        Interface0DIterator. The result is obtained by querying the depth
        buffer on a window around that point.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The average depth around the pointed Interface0D.
                :rtype: float
        """
        ...

class LocalAverageDepthF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `LocalAverageDepthF1D`"""

    def __init__(
        self, sigma: float, integration_type: freestyle.types.IntegrationType = None
    ):
        """Builds a LocalAverageDepthF1D object.

                :param sigma: The sigma used in DensityF0D and determining the window
        size used in each density query.
                :type sigma: float
                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns the average depth evaluated for an Interface1D. The average
        depth is evaluated for a set of points along the Interface1D (using
        the `freestyle.functions.LocalAverageDepthF0D` functor) with a
        user-defined sampling and then integrated into a single value using a
        user-defined integration method.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: The average depth evaluated for the Interface1D.
                :rtype: float
        """
        ...

class MaterialF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DMaterial` > `MaterialF0D`"""

    def __init__(self):
        """Builds a MaterialF0D object."""
        ...

    def __call__(
        self, it: freestyle.types.Interface0DIterator
    ) -> freestyle.types.Material:
        """Returns the material of the object evaluated at the
        `freestyle.types.Interface0D` pointed by the
        Interface0DIterator. This evaluation can be ambiguous (in the case of
        a `freestyle.types.TVertex` for example. This functor tries to
        remove this ambiguity using the context offered by the 1D element to
        which the Interface0DIterator belongs to and by arbitrary choosing the
        material of the face that lies on its left when following the 1D
        element if there are two different materials on each side of the
        point. However, there still can be problematic cases, and the user
        willing to deal with this cases in a specific way should implement its
        own getMaterial functor.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The material of the object evaluated at the pointed
        Interface0D.
                :rtype: freestyle.types.Material
        """
        ...

class Normal2DF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DVec2f` > `Normal2DF0D`"""

    def __init__(self):
        """Builds a Normal2DF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> mathutils.Vector:
        """Returns a two-dimensional vector giving the normalized 2D normal to
        the 1D element to which the `freestyle.types.Interface0D`
        pointed by the Interface0DIterator belongs. The normal is evaluated
        at the pointed Interface0D.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The 2D normal of the 1D element evaluated at the pointed
        Interface0D.
                :rtype: mathutils.Vector
        """
        ...

class Normal2DF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DVec2f` > `Normal2DF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a Normal2DF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> mathutils.Vector:
        """Returns the 2D normal for the Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The 2D normal for the Interface1D.
        :rtype: mathutils.Vector
        """
        ...

class Orientation2DF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DVec2f` > `Orientation2DF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds an Orientation2DF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> mathutils.Vector:
        """Returns the 2D orientation of the Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The 2D orientation of the Interface1D.
        :rtype: mathutils.Vector
        """
        ...

class Orientation3DF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DVec3f` > `Orientation3DF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds an Orientation3DF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> mathutils.Vector:
        """Returns the 3D orientation of the Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        :return: The 3D orientation of the Interface1D.
        :rtype: mathutils.Vector
        """
        ...

class QuantitativeInvisibilityF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DUnsigned` > `QuantitativeInvisibilityF0D`"""

    def __init__(self):
        """Builds a QuantitativeInvisibilityF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> int:
        """Returns the quantitative invisibility of the
        `freestyle.types.Interface0D` pointed by the
        Interface0DIterator. This evaluation can be ambiguous (in the case of
        a `freestyle.types.TVertex` for example). This functor tries
        to remove this ambiguity using the context offered by the 1D element
        to which the Interface0D belongs to. However, there still can be
        problematic cases, and the user willing to deal with this cases in a
        specific way should implement its own getQIF0D functor.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The quantitative invisibility of the pointed Interface0D.
                :rtype: int
        """
        ...

class QuantitativeInvisibilityF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DUnsigned` > `QuantitativeInvisibilityF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a QuantitativeInvisibilityF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> int:
        """Returns the Quantitative Invisibility of an Interface1D element. If
        the Interface1D is a `freestyle.types.ViewEdge`, then there is
        no ambiguity concerning the result. But, if the Interface1D results
        of a chaining (chain, stroke), then it might be made of several 1D
        elements of different Quantitative Invisibilities.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: The Quantitative Invisibility of the Interface1D.
                :rtype: int
        """
        ...

class ReadCompleteViewMapPixelF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DFloat` > `ReadCompleteViewMapPixelF0D`"""

    def __init__(self, level: int):
        """Builds a ReadCompleteViewMapPixelF0D object.

                :param level: The level of the pyramid from which the pixel must be
        read.
                :type level: int
        """
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Reads a pixel in one of the level of the complete viewmap.

        :param it: An Interface0DIterator object.
        :type it: freestyle.types.Interface0DIterator
        :return: A pixel in one of the level of the complete viewmap.
        :rtype: float
        """
        ...

class ReadMapPixelF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DFloat` > `ReadMapPixelF0D`"""

    def __init__(self, map_name: str, level: int):
        """Builds a ReadMapPixelF0D object.

                :param map_name: The name of the map to be read.
                :type map_name: str
                :param level: The level of the pyramid from which the pixel must be
        read.
                :type level: int
        """
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Reads a pixel in a map.

        :param it: An Interface0DIterator object.
        :type it: freestyle.types.Interface0DIterator
        :return: A pixel in a map.
        :rtype: float
        """
        ...

class ReadSteerableViewMapPixelF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DFloat` > `ReadSteerableViewMapPixelF0D`"""

    def __init__(self, orientation: int, level: int):
        """Builds a ReadSteerableViewMapPixelF0D object.

                :param orientation: The integer belonging to [0, 4] indicating the
        orientation (E, NE, N, NW) we are interested in.
                :type orientation: int
                :param level: The level of the pyramid from which the pixel must be
        read.
                :type level: int
        """
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Reads a pixel in one of the level of one of the steerable viewmaps.

        :param it: An Interface0DIterator object.
        :type it: freestyle.types.Interface0DIterator
        :return: A pixel in one of the level of one of the steerable viewmaps.
        :rtype: float
        """
        ...

class ShapeIdF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DId` > `ShapeIdF0D`"""

    def __init__(self):
        """Builds a ShapeIdF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> freestyle.types.Id:
        """Returns the `freestyle.types.Id` of the Shape the
        `freestyle.types.Interface0D` pointed by the
        Interface0DIterator belongs to. This evaluation can be ambiguous (in
        the case of a `freestyle.types.TVertex` for example). This
        functor tries to remove this ambiguity using the context offered by
        the 1D element to which the Interface0DIterator belongs to. However,
        there still can be problematic cases, and the user willing to deal
        with this cases in a specific way should implement its own
        getShapeIdF0D functor.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The Id of the Shape the pointed Interface0D belongs to.
                :rtype: freestyle.types.Id
        """
        ...

class TimeStampF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DVoid` > `TimeStampF1D`"""

    def __init__(self):
        """Builds a TimeStampF1D object."""
        ...

    def __call__(self, inter: freestyle.types.Interface1D):
        """Returns the time stamp of the Interface1D.

        :param inter: An Interface1D object.
        :type inter: freestyle.types.Interface1D
        """
        ...

class VertexOrientation2DF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DVec2f` > `VertexOrientation2DF0D`"""

    def __init__(self):
        """Builds a VertexOrientation2DF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> mathutils.Vector:
        """Returns a two-dimensional vector giving the 2D oriented tangent to the
        1D element to which the `freestyle.types.Interface0D` pointed
        by the Interface0DIterator belongs. The 2D oriented tangent is
        evaluated at the pointed Interface0D.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The 2D oriented tangent to the 1D element evaluated at the
        pointed Interface0D.
                :rtype: mathutils.Vector
        """
        ...

class VertexOrientation3DF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DVec3f` > `VertexOrientation3DF0D`"""

    def __init__(self):
        """Builds a VertexOrientation3DF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> mathutils.Vector:
        """Returns a three-dimensional vector giving the 3D oriented tangent to
        the 1D element to which the `freestyle.types.Interface0D`
        pointed by the Interface0DIterator belongs. The 3D oriented tangent
        is evaluated at the pointed Interface0D.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The 3D oriented tangent to the 1D element evaluated at the
        pointed Interface0D.
                :rtype: mathutils.Vector
        """
        ...

class ZDiscontinuityF0D:
    """Class hierarchy: `freestyle.types.UnaryFunction0D` > `freestyle.types.UnaryFunction0DDouble` > `ZDiscontinuityF0D`"""

    def __init__(self):
        """Builds a ZDiscontinuityF0D object."""
        ...

    def __call__(self, it: freestyle.types.Interface0DIterator) -> float:
        """Returns a real value giving the distance between the
        `freestyle.types.Interface0D` pointed by the
        Interface0DIterator and the shape that lies behind (occludee). This
        distance is evaluated in the camera space and normalized between 0 and
        1. Therefore, if no object is occluded by the shape to which the
        Interface0D belongs to, 1 is returned.

                :param it: An Interface0DIterator object.
                :type it: freestyle.types.Interface0DIterator
                :return: The normalized distance between the pointed Interface0D
        and the occludee.
                :rtype: float
        """
        ...

class ZDiscontinuityF1D:
    """Class hierarchy: `freestyle.types.UnaryFunction1D` > `freestyle.types.UnaryFunction1DDouble` > `ZDiscontinuityF1D`"""

    def __init__(self, integration_type: freestyle.types.IntegrationType = None):
        """Builds a ZDiscontinuityF1D object.

                :param integration_type: The integration method used to compute a single value
        from a set of values.
                :type integration_type: freestyle.types.IntegrationType
        """
        ...

    def __call__(self, inter: freestyle.types.Interface1D) -> float:
        """Returns a real value giving the distance between an Interface1D
        and the shape that lies behind (occludee). This distance is
        evaluated in the camera space and normalized between 0 and 1.
        Therefore, if no object is occluded by the shape to which the
        Interface1D belongs to, 1 is returned.

                :param inter: An Interface1D object.
                :type inter: freestyle.types.Interface1D
                :return: The normalized distance between the Interface1D and the occludee.
                :rtype: float
        """
        ...

class pyCurvilinearLengthF0D: ...

class pyDensityAnisotropyF0D:
    """Estimates the anisotropy of density."""

    ...

class pyDensityAnisotropyF1D: ...
class pyGetInverseProjectedZF1D: ...
class pyGetSquareInverseProjectedZF1D: ...
class pyInverseCurvature2DAngleF0D: ...
class pyViewMapGradientNormF0D: ...
class pyViewMapGradientNormF1D: ...

class pyViewMapGradientVectorF0D:
    """Returns the gradient vector for a pixel."""

    def __init__(self, level: int):
        """Builds a pyViewMapGradientVectorF0D object.

        :param level: the level at which to compute the gradient
        :type level: int
        """
        ...



import typing
import collections.abc
import bpy.types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def denoise_animation(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    input_filepath: str | typing.Any = "",
    output_filepath: str | typing.Any = "",
):
    """Denoise rendered animation sequence using current scene and view layer settings. Requires denoising data passes and output to OpenEXR multilayer files

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param input_filepath: Input Filepath, File path for image to denoise. If not specified, uses the render file path and frame range from the scene
    :type input_filepath: str | typing.Any
    :param output_filepath: Output Filepath, If not specified, renders will be denoised in-place
    :type output_filepath: str | typing.Any
    """

    ...

def merge_images(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
    *,
    input_filepath1: str | typing.Any = "",
    input_filepath2: str | typing.Any = "",
    output_filepath: str | typing.Any = "",
):
    """Combine OpenEXR multi-layer images rendered with different sample ranges into one image with reduced noise

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    :param input_filepath1: Input Filepath, File path for image to merge
    :type input_filepath1: str | typing.Any
    :param input_filepath2: Input Filepath, File path for image to merge
    :type input_filepath2: str | typing.Any
    :param output_filepath: Output Filepath, File path for merged image
    :type output_filepath: str | typing.Any
    """

    ...

def use_shading_nodes(
    override_context: bpy.types.Context | dict[str, typing.Any] | None = None,
    execution_context: int | str | None = None,
    undo: bool | None = None,
):
    """Enable nodes on a material, world or light

    :type override_context: bpy.types.Context | dict[str, typing.Any] | None
    :type execution_context: int | str | None
    :type undo: bool | None
    """

    ...



import typing
import collections.abc
from . import helpers

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def class_filter(cls_parent, kw): ...
def ui_draw_filter_register(
    ui_ignore_classes=None,
    ui_ignore_operator=None,
    ui_ignore_property=None,
    ui_ignore_menu=None,
    ui_ignore_label=None,
): ...
def ui_draw_filter_unregister(ui_ignore_store): ...



"""
This module provides access to selection.

"""

import typing
import collections.abc

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def load_id(id):
    """Set the selection ID.

    :param id: Number (32-bit uint).
    """

    ...



import typing
import collections.abc
import bpy_types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class ToolActivePanelHelper:
    bl_label: typing.Any

    def draw(self, context):
        """

        :param context:
        """
        ...

class ToolDef:
    cursor: typing.Any
    data_block: typing.Any
    description: typing.Any
    draw_cursor: typing.Any
    draw_settings: typing.Any
    icon: typing.Any
    idname: typing.Any
    keymap: typing.Any
    label: typing.Any
    operator: typing.Any
    options: typing.Any
    widget: typing.Any
    widget_properties: typing.Any

    def count(self, value):
        """

        :param value:
        """
        ...

    def from_dict(self, kw_args):
        """

        :param kw_args:
        """
        ...

    def from_fn(self, fn):
        """

        :param fn:
        """
        ...

    def index(self, value, start=0, stop=9223372036854775807):
        """

        :param value:
        :param start:
        :param stop:
        """
        ...

class ToolSelectPanelHelper:
    def draw(self, context):
        """

        :param context:
        """
        ...

    def draw_active_tool_fallback(
        self, context, layout, tool, is_horizontal_layout=False
    ):
        """

        :param context:
        :param layout:
        :param tool:
        :param is_horizontal_layout:
        """
        ...

    def draw_active_tool_header(
        self, context, layout, show_tool_icon_always=False, tool_key=None
    ):
        """

        :param context:
        :param layout:
        :param show_tool_icon_always:
        :param tool_key:
        """
        ...

    def draw_cls(self, layout, context, detect_layout=True, scale_y=1.75):
        """

        :param layout:
        :param context:
        :param detect_layout:
        :param scale_y:
        """
        ...

    def draw_fallback_tool_items(self, layout, context):
        """

        :param layout:
        :param context:
        """
        ...

    def draw_fallback_tool_items_for_pie_menu(self, layout, context):
        """

        :param layout:
        :param context:
        """
        ...

    def keymap_ui_hierarchy(self, context_mode):
        """

        :param context_mode:
        """
        ...

    def register(self): ...
    def register_ensure(self): ...
    def tool_active_from_context(self, context):
        """

        :param context:
        """
        ...

    def tools_all(self): ...
    def tools_from_context(self, context, mode=None):
        """

        :param context:
        :param mode:
        """
        ...

class WM_MT_toolsystem_submenu(bpy_types.Menu, bpy_types._GenericUI):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def append(self, draw_func):
        """

        :param draw_func:
        """
        ...

    def as_pointer(self): ...
    def bl_rna_get_subclass(self): ...
    def bl_rna_get_subclass_py(self): ...
    def draw(self, context):
        """

        :param context:
        """
        ...

    def draw_collapsible(self, context, layout):
        """

        :param context:
        :param layout:
        """
        ...

    def draw_preset(self, _context):
        """

        :param _context:
        """
        ...

    def driver_add(self): ...
    def driver_remove(self): ...
    def get(self): ...
    def id_properties_clear(self): ...
    def id_properties_ensure(self): ...
    def id_properties_ui(self): ...
    def is_extended(self): ...
    def is_property_hidden(self): ...
    def is_property_overridable_library(self): ...
    def is_property_readonly(self): ...
    def is_property_set(self): ...
    def items(self): ...
    def keyframe_delete(self): ...
    def keyframe_insert(self): ...
    def keys(self): ...
    def path_from_id(self): ...
    def path_menu(
        self,
        searchpaths,
        operator,
        props_default=None,
        prop_filepath="filepath",
        filter_ext=None,
        filter_path=None,
        display_name=None,
        add_operator=None,
        add_operator_props=None,
    ):
        """

        :param searchpaths:
        :param operator:
        :param props_default:
        :param prop_filepath:
        :param filter_ext:
        :param filter_path:
        :param display_name:
        :param add_operator:
        :param add_operator_props:
        """
        ...

    def path_resolve(self): ...
    def pop(self): ...
    def prepend(self, draw_func):
        """

        :param draw_func:
        """
        ...

    def property_overridable_library_set(self): ...
    def property_unset(self): ...
    def remove(self, draw_func):
        """

        :param draw_func:
        """
        ...

    def type_recast(self): ...
    def values(self): ...

def activate_by_id(context, space_type, idname, as_fallback=False): ...
def activate_by_id_or_cycle(
    context, space_type, idname, offset=1, as_fallback=False
): ...
def description_from_id(context, space_type, idname, use_operator=True): ...
def item_from_flat_index(context, space_type, index): ...
def item_from_id(context, space_type, idname): ...
def item_from_id_active(context, space_type, idname): ...
def item_from_id_active_with_group(context, space_type, idname): ...
def item_from_index_active(context, space_type, index): ...
def item_group_from_id(context, space_type, idname, coerce=False): ...
def keymap_from_id(context, space_type, idname): ...



"""
This module provides data types of view map components (0D and 1D
elements), base classes for defining line stylization rules
(predicates, functions, chaining iterators, and stroke shaders),
as well as helper functions for style module writing.

freestyle.types.rst
freestyle.predicates.rst
freestyle.functions.rst
freestyle.chainingiterators.rst
freestyle.shaders.rst
freestyle.utils.rst

:maxdepth: 1
:caption: Submodules

"""

import typing
import collections.abc
from . import chainingiterators
from . import functions
from . import predicates
from . import shaders
from . import types
from . import utils

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")



import typing
import collections.abc
import bpy.types
import bpy_types

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

class GenericUIListOperator:
    """Mix-in class containing functionality shared by operators
    that deal with managing Blender list entries.
    """

    bl_options: typing.Any

    def get_active_index(self, context):
        """

        :param context:
        """
        ...

    def get_list(self, context):
        """

        :param context:
        """
        ...

    def set_active_index(self, context, index):
        """

        :param context:
        :param index:
        """
        ...

class UILIST_OT_entry_add(GenericUIListOperator, bpy_types.Operator):
    """Add an entry to the list after the current active item"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def as_keywords(self, ignore=()):
        """Return a copy of the properties as a dictionary

        :param ignore:
        """
        ...

    def as_pointer(self) -> int:
        """Returns the memory address which holds a pointer to Blender's internal data

        :return: int (memory address).
        :rtype: int
        """
        ...

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def driver_add(self) -> bpy.types.FCurve:
        """Adds driver(s) to the given property

        :return: The driver(s) added.
        :rtype: bpy.types.FCurve
        """
        ...

    def driver_remove(self) -> bool:
        """Remove driver(s) from the given property

        :return: Success of driver removal.
        :rtype: bool
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def get(self):
        """Returns the value of the custom property assigned to key or default
        when not found (matches Python's dictionary function of the same name).

        """
        ...

    def get_active_index(self, context):
        """

        :param context:
        """
        ...

    def get_list(self, context):
        """

        :param context:
        """
        ...

    def id_properties_clear(self):
        """

        :return: Remove the parent group for an RNA struct's custom IDProperties.
        """
        ...

    def id_properties_ensure(self):
        """

        :return: the parent group for an RNA struct's custom IDProperties.
        """
        ...

    def id_properties_ui(self):
        """

        :return: Return an object used to manage an IDProperty's UI data.
        """
        ...

    def is_property_hidden(self) -> bool:
        """Check if a property is hidden.

        :return: True when the property is hidden.
        :rtype: bool
        """
        ...

    def is_property_overridable_library(self) -> bool:
        """Check if a property is overridable.

        :return: True when the property is overridable.
        :rtype: bool
        """
        ...

    def is_property_readonly(self) -> bool:
        """Check if a property is readonly.

        :return: True when the property is readonly (not writable).
        :rtype: bool
        """
        ...

    def is_property_set(self) -> bool:
        """Check if a property is set, use for testing operator properties.

        :return: True when the property has been set.
        :rtype: bool
        """
        ...

    def items(self):
        """Returns the items of this objects custom properties (matches Python's
        dictionary function of the same name).

                :return: custom property key, value pairs.
        """
        ...

    def keyframe_delete(self) -> bool:
        """Remove a keyframe from this properties fcurve.

        :return: Success of keyframe deletion.
        :rtype: bool
        """
        ...

    def keyframe_insert(self) -> bool:
        """Insert a keyframe on the property given, adding fcurves and animation data when necessary.

        :return: Success of keyframe insertion.
        :rtype: bool
        """
        ...

    def keys(self):
        """Returns the keys of this objects custom properties (matches Python's
        dictionary function of the same name).

                :return: custom property keys.
        """
        ...

    def path_from_id(self) -> str:
        """Returns the data path from the ID to this object (string).

                :return: The path from `bpy.types.bpy_struct.id_data`
        to this struct and property (when given).
                :rtype: str
        """
        ...

    def path_resolve(self):
        """Returns the property from the path, raise an exception when not found."""
        ...

    def poll_message_set(self):
        """Set the message to show in the tool-tip when poll fails.When message is callable, additional user defined positional arguments are passed to the message function."""
        ...

    def pop(self):
        """Remove and return the value of the custom property assigned to key or default
        when not found (matches Python's dictionary function of the same name).

        """
        ...

    def property_overridable_library_set(self) -> bool:
        """Define a property as overridable or not (only for custom properties!).

        :return: True when the overridable status of the property was successfully set.
        :rtype: bool
        """
        ...

    def property_unset(self):
        """Unset a property, will use default value afterward."""
        ...

    def set_active_index(self, context, index):
        """

        :param context:
        :param index:
        """
        ...

    def type_recast(self):
        """Return a new instance, this is needed because types
        such as textures can be changed at runtime.

                :return: a new instance of this object with the type initialized again.
        """
        ...

    def values(self):
        """Returns the values of this objects custom properties (matches Python's
        dictionary function of the same name).

                :return: custom property values.
        """
        ...

class UILIST_OT_entry_move(GenericUIListOperator, bpy_types.Operator):
    """Move an entry in the list up or down"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def as_keywords(self, ignore=()):
        """Return a copy of the properties as a dictionary

        :param ignore:
        """
        ...

    def as_pointer(self) -> int:
        """Returns the memory address which holds a pointer to Blender's internal data

        :return: int (memory address).
        :rtype: int
        """
        ...

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def driver_add(self) -> bpy.types.FCurve:
        """Adds driver(s) to the given property

        :return: The driver(s) added.
        :rtype: bpy.types.FCurve
        """
        ...

    def driver_remove(self) -> bool:
        """Remove driver(s) from the given property

        :return: Success of driver removal.
        :rtype: bool
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def get(self):
        """Returns the value of the custom property assigned to key or default
        when not found (matches Python's dictionary function of the same name).

        """
        ...

    def get_active_index(self, context):
        """

        :param context:
        """
        ...

    def get_list(self, context):
        """

        :param context:
        """
        ...

    def id_properties_clear(self):
        """

        :return: Remove the parent group for an RNA struct's custom IDProperties.
        """
        ...

    def id_properties_ensure(self):
        """

        :return: the parent group for an RNA struct's custom IDProperties.
        """
        ...

    def id_properties_ui(self):
        """

        :return: Return an object used to manage an IDProperty's UI data.
        """
        ...

    def is_property_hidden(self) -> bool:
        """Check if a property is hidden.

        :return: True when the property is hidden.
        :rtype: bool
        """
        ...

    def is_property_overridable_library(self) -> bool:
        """Check if a property is overridable.

        :return: True when the property is overridable.
        :rtype: bool
        """
        ...

    def is_property_readonly(self) -> bool:
        """Check if a property is readonly.

        :return: True when the property is readonly (not writable).
        :rtype: bool
        """
        ...

    def is_property_set(self) -> bool:
        """Check if a property is set, use for testing operator properties.

        :return: True when the property has been set.
        :rtype: bool
        """
        ...

    def items(self):
        """Returns the items of this objects custom properties (matches Python's
        dictionary function of the same name).

                :return: custom property key, value pairs.
        """
        ...

    def keyframe_delete(self) -> bool:
        """Remove a keyframe from this properties fcurve.

        :return: Success of keyframe deletion.
        :rtype: bool
        """
        ...

    def keyframe_insert(self) -> bool:
        """Insert a keyframe on the property given, adding fcurves and animation data when necessary.

        :return: Success of keyframe insertion.
        :rtype: bool
        """
        ...

    def keys(self):
        """Returns the keys of this objects custom properties (matches Python's
        dictionary function of the same name).

                :return: custom property keys.
        """
        ...

    def path_from_id(self) -> str:
        """Returns the data path from the ID to this object (string).

                :return: The path from `bpy.types.bpy_struct.id_data`
        to this struct and property (when given).
                :rtype: str
        """
        ...

    def path_resolve(self):
        """Returns the property from the path, raise an exception when not found."""
        ...

    def poll_message_set(self):
        """Set the message to show in the tool-tip when poll fails.When message is callable, additional user defined positional arguments are passed to the message function."""
        ...

    def pop(self):
        """Remove and return the value of the custom property assigned to key or default
        when not found (matches Python's dictionary function of the same name).

        """
        ...

    def property_overridable_library_set(self) -> bool:
        """Define a property as overridable or not (only for custom properties!).

        :return: True when the overridable status of the property was successfully set.
        :rtype: bool
        """
        ...

    def property_unset(self):
        """Unset a property, will use default value afterward."""
        ...

    def set_active_index(self, context, index):
        """

        :param context:
        :param index:
        """
        ...

    def type_recast(self):
        """Return a new instance, this is needed because types
        such as textures can be changed at runtime.

                :return: a new instance of this object with the type initialized again.
        """
        ...

    def values(self):
        """Returns the values of this objects custom properties (matches Python's
        dictionary function of the same name).

                :return: custom property values.
        """
        ...

class UILIST_OT_entry_remove(GenericUIListOperator, bpy_types.Operator):
    """Remove the selected entry from the list"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def as_keywords(self, ignore=()):
        """Return a copy of the properties as a dictionary

        :param ignore:
        """
        ...

    def as_pointer(self) -> int:
        """Returns the memory address which holds a pointer to Blender's internal data

        :return: int (memory address).
        :rtype: int
        """
        ...

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """
        ...

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """
        ...

    def driver_add(self) -> bpy.types.FCurve:
        """Adds driver(s) to the given property

        :return: The driver(s) added.
        :rtype: bpy.types.FCurve
        """
        ...

    def driver_remove(self) -> bool:
        """Remove driver(s) from the given property

        :return: Success of driver removal.
        :rtype: bool
        """
        ...

    def execute(self, context):
        """

        :param context:
        """
        ...

    def get(self):
        """Returns the value of the custom property assigned to key or default
        when not found (matches Python's dictionary function of the same name).

        """
        ...

    def get_active_index(self, context):
        """

        :param context:
        """
        ...

    def get_list(self, context):
        """

        :param context:
        """
        ...

    def id_properties_clear(self):
        """

        :return: Remove the parent group for an RNA struct's custom IDProperties.
        """
        ...

    def id_properties_ensure(self):
        """

        :return: the parent group for an RNA struct's custom IDProperties.
        """
        ...

    def id_properties_ui(self):
        """

        :return: Return an object used to manage an IDProperty's UI data.
        """
        ...

    def is_property_hidden(self) -> bool:
        """Check if a property is hidden.

        :return: True when the property is hidden.
        :rtype: bool
        """
        ...

    def is_property_overridable_library(self) -> bool:
        """Check if a property is overridable.

        :return: True when the property is overridable.
        :rtype: bool
        """
        ...

    def is_property_readonly(self) -> bool:
        """Check if a property is readonly.

        :return: True when the property is readonly (not writable).
        :rtype: bool
        """
        ...

    def is_property_set(self) -> bool:
        """Check if a property is set, use for testing operator properties.

        :return: True when the property has been set.
        :rtype: bool
        """
        ...

    def items(self):
        """Returns the items of this objects custom properties (matches Python's
        dictionary function of the same name).

                :return: custom property key, value pairs.
        """
        ...

    def keyframe_delete(self) -> bool:
        """Remove a keyframe from this properties fcurve.

        :return: Success of keyframe deletion.
        :rtype: bool
        """
        ...

    def keyframe_insert(self) -> bool:
        """Insert a keyframe on the property given, adding fcurves and animation data when necessary.

        :return: Success of keyframe insertion.
        :rtype: bool
        """
        ...

    def keys(self):
        """Returns the keys of this objects custom properties (matches Python's
        dictionary function of the same name).

                :return: custom property keys.
        """
        ...

    def path_from_id(self) -> str:
        """Returns the data path from the ID to this object (string).

                :return: The path from `bpy.types.bpy_struct.id_data`
        to this struct and property (when given).
                :rtype: str
        """
        ...

    def path_resolve(self):
        """Returns the property from the path, raise an exception when not found."""
        ...

    def poll_message_set(self):
        """Set the message to show in the tool-tip when poll fails.When message is callable, additional user defined positional arguments are passed to the message function."""
        ...

    def pop(self):
        """Remove and return the value of the custom property assigned to key or default
        when not found (matches Python's dictionary function of the same name).

        """
        ...

    def property_overridable_library_set(self) -> bool:
        """Define a property as overridable or not (only for custom properties!).

        :return: True when the overridable status of the property was successfully set.
        :rtype: bool
        """
        ...

    def property_unset(self):
        """Unset a property, will use default value afterward."""
        ...

    def set_active_index(self, context, index):
        """

        :param context:
        :param index:
        """
        ...

    def type_recast(self):
        """Return a new instance, this is needed because types
        such as textures can be changed at runtime.

                :return: a new instance of this object with the type initialized again.
        """
        ...

    def values(self):
        """Returns the values of this objects custom properties (matches Python's
        dictionary function of the same name).

                :return: custom property values.
        """
        ...

def draw_ui_list(
    layout: bpy.types.UILayout,
    context: bpy.types.Context,
    class_name: str = "UI_UL_list",
    unique_id: str = None,
    list_path: str = None,
    active_index_path: str = None,
    insertion_operators: bool = True,
    move_operators: str = True,
    menu_class_name: str = "",
    kwargs=None,
):
    """Draw a UIList with Add/Remove/Move buttons and a menu.Additional keyword arguments are passed to `UIList.template_list`.

        :param layout: UILayout to draw the list in.
        :type layout: bpy.types.UILayout
        :param context: Blender context to get the list data from.
        :type context: bpy.types.Context
        :param class_name: Name of the UIList class to draw. The default is the UIList class that ships with Blender.
        :type class_name: str
        :param unique_id: Unique identifier to differentiate this from other UI lists.
        :type unique_id: str
        :param list_path: Data path of the list relative to context, eg. "object.vertex_groups".
        :type list_path: str
        :param active_index_path: Data path of the list active index integer relative to context,
    eg. "object.vertex_groups.active_index".
        :type active_index_path: str
        :param insertion_operators: Whether to draw Add/Remove buttons.
        :type insertion_operators: bool
        :param move_operators: Whether to draw Move Up/Down buttons.
        :type move_operators: str
        :param menu_class_name: Identifier of a Menu that should be drawn as a drop-down.
        :type menu_class_name: str
        :return: The right side column.
    """

    ...



"""

--------------------

```../examples/bpy.app.timers.1.py```


--------------------

```../examples/bpy.app.timers.2.py```


--------------------

```../examples/bpy.app.timers.3.py```


--------------------

```../examples/bpy.app.timers.4.py```


--------------------

You should never modify Blender data at arbitrary points in time in separate threads.
However you can use a queue to collect all the actions that should be executed when Blender is in the right state again.
Pythons queue.Queue can be used here, because it implements the required locking semantics.

```../examples/bpy.app.timers.5.py```

"""

import typing
import collections.abc

GenericType1 = typing.TypeVar("GenericType1")
GenericType2 = typing.TypeVar("GenericType2")

def is_registered(function: collections.abc.Callable[[], float]) -> bool:
    """Check if this function is registered as a timer.

    :param function: Function to check.
    :type function: collections.abc.Callable[[], float]
    :return: True when this function is registered, otherwise False.
    :rtype: bool
    """

    ...

def register(function: collections.abc.Callable[[], float]):
    """Add a new function that will be called after the specified amount of seconds.
    The function gets no arguments and is expected to return either None or a float.
    If None is returned, the timer will be unregistered.
    A returned number specifies the delay until the function is called again.
    functools.partial can be used to assign some parameters.

        :param function: The function that should called.
        :type function: collections.abc.Callable[[], float]
    """

    ...

def unregister(function: collections.abc.Callable[[], float]):
    """Unregister timer.

    :param function: Function to unregister.
    :type function: collections.abc.Callable[[], float]
    """

    ...
